@@ -134,8 +134,13 @@ def qq_download(url, output_dir='.', merge=True, info_only=False, **kwargs):
     if 'v.qq.com/page' in url:
         # for URLs like this:
         # http://v.qq.com/page/k/9/7/k0194pwgw97.html
-        content = get_html(url)
-        url = match1(content,r'window\.location\.href="(.*?)"')
+        new_url = url_locations([url])[0]
+        if url == new_url:
+            #redirect in js?
+            content = get_content(url)
+            url = match1(content,r'window\.location\.href="(.*?)"')
+        else:
+            url = new_url
 
     if 'kuaibao.qq.com' in url or re.match(r'http://daxue.qq.com/content/content/id/\d+', url):
         content = get_html(url)
